// spinock mutex je hitrejši - niti so aktivne in čakajo na vstop
// fair: nit, ki je bila prva v vrsti, prva dobi vstop

#include <stdio.h>
#include <pthread.h>
#include <omp.h>

#define N 250000000
#define NTHREADS 2

double pi = 0.0;

pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
pthread_spinlock_t slock;

void *thread_sum(void *arg)
{
	long i, sign;
	int *myrank = (int *)arg;
	long mystart = (int)(N / (double)NTHREADS * *myrank);
	long myend = (int)(N / (double)NTHREADS * (*myrank + 1));

	sign = 1;
	for (i = mystart; i<myend; i++, sign = -sign)
	{
		//pthread_mutex_lock(&lock);
		pthread_spin_lock(&slock);
		pi += sign * 4.0 / (2 * i + 1);
		//pthread_mutex_unlock(&lock);
		pthread_spin_unlock(&slock);
	}

	return NULL;
}

int main(void)
{
	int i;
	pthread_t t[NTHREADS];
	int p[NTHREADS];

	double dt = omp_get_wtime();

	pthread_spin_init(&slock, PTHREAD_PROCESS_PRIVATE);

	for (i = 0; i<NTHREADS; i++)
	{
		p[i] = i;
		pthread_create(&t[i], NULL, thread_sum, (void *)&p[i]);
	}

	for (i = 0; i<NTHREADS; i++)
		pthread_join(t[i], NULL);

	pthread_spin_destroy(&slock);

	dt = omp_get_wtime() - dt;

	printf("pi = %lf (%lf)\n", pi, dt);

	return 0;
}
